/***************************************************************
 * Name:      ScanThread.cpp
 * Purpose:   Code for the thread wich scan directory
 * Author:    Jackie Team (jackie.team@gmail.com)
 * Created:   2007-12-16
 * Copyright: Jackie Team (http://musicorganizer.free.fr)
 * License: GNU GPL
 **************************************************************/

#ifdef WX_PRECOMP
#include "wx_pch.h"
#endif

#ifdef __BORLANDC__
#pragma hdrstop
#endif //__BORLANDC__

#include "components.h"


void ScanThread::walkThrough(std::string chemin, bool r){

    DIR * pdir;
    struct dirent *pent;
    std::string name;
    //wxString name2;
    pdir=opendir(chemin.c_str());
    if(!pdir)
       return;
    while ((pent=readdir(pdir))){
        name=pent->d_name;
        if(stop==true)
            throw 1;
        if(name!=".." && name !="."){
            name=chemin+SEPARATOR+name;
            std::string namel=name;
            std::transform(name.begin(), name.end(), namel.begin(), tolower);
            if(name.length()>3 && namel.substr(namel.length()-4,4)==".mp3"){//if it's an mp3 file
                file->push_back(name);
            }else if(r)
                walkThrough(name,r);
        }
    }
    closedir(pdir);

}

ScanThread::ScanThread(wxGenericDirCtrl* a,std::string c,std::vector<std::string>* f,bool rec):wxThread(){
    stop=false;
    chemin=c;
    ma=a;
    file=f;
    recursive=rec;
}

void *ScanThread::Entry(){
    try{
        walkThrough(chemin,recursive);
    }catch(...){}
    return NULL;
}

void ScanThread::OnExit(){
    wxCommandEvent event( wxEVT_COMMAND_MENU_SELECTED, THREAD);
    event.SetInt(-1); // that's all
    wxPostEvent( ma, event );
}

void ScanThread::Stop(){
    stop=true;
}

/* ********************************************************************************************** */
CopyMoveThread::CopyMoveThread(wxListCtrl* l,wxNotebook* b,wxString& d,bool move){
    ml=l;
    mb=b;
    mv=move;
    prog=0.;
    errors=0;
    dossier=d;
}

void CopyMoveThread::OnExit(){
    wxCommandEvent event( wxEVT_COMMAND_MENU_SELECTED, ID_THREAD);
    event.SetInt(errors); // that's all
    wxPostEvent( mb, event );
}

void *CopyMoveThread::Entry(){
    MaListe* l=(MaListe*)ml;
    MonBook* b=(MonBook*)mb;
    long AncienneSel = l->getSelection(-2);
    for (int h=0 ; h<l->GetSelectedItemCount() ; h++){
        wxString artiste = l->getSelRowText(2,AncienneSel);
        wxString album = l->getSelRowText(3,AncienneSel);
        wxString annee = l->getSelRowText(4,AncienneSel);
        wxString genre = l->getSelRowText(5,AncienneSel);
        wxString SaisieStr,path,path_ori;
        if (artiste == _T(""))
            artiste = _("Unknown artist");
        if (album == _T(""))
            album = _("Unknown album");
        if (annee == _T(""))
            annee = _("Unknown year");
        if (genre == _T(""))
            genre = _("Unknown genre");
        artiste.Replace(_T("/"),_T("_"));
        album.Replace(_T("/"),_T("_"));
        annee.Replace(_T("/"),_T("_"));
        genre.Replace(_T("/"),_T("_"));
        SaisieStr = b->getComboOrg()->GetValue();
        SaisieStr.Replace(_T("%1"), artiste.Trim());
        SaisieStr.Replace(_T("%2"), album.Trim());
        SaisieStr.Replace(_T("%3"), annee.Trim());
        SaisieStr.Replace(_T("%4"), genre.Trim());
        wxString rchar=OptionsHandler::getOption(_T("replace_char"));
        if(rchar==_T(""))
            rchar=_T("_");
        SaisieStr.Replace(_T("\""),rchar);
        b->cleanStr(SaisieStr,rchar);


        path=dossier;
        wxStringTokenizer tkz(SaisieStr,wxT("/"));//split la chaine sur le caractre '/'
        try{
            wxRegEx backSlashes(wxT("\\\\\\*"));
            while(tkz.HasMoreTokens() ){
                wxString token = tkz.GetNextToken();
                path.Printf(path+wxT("%c")+token,SEPARATOR);
                backSlashes.Replace(&path,wxString(_T("\\")),0);
                //DirPortable::mkdirPortable((char const*)path.mb_str(wxConvISO8859_1));
                wxLogNull nolog;
                wxMkdir(path);
            }

            path.Printf(path+wxT("%c")+l->getSelRowText(0,AncienneSel),SEPARATOR);
            backSlashes.Replace(&path,wxString(_T("\\")),0);

            path_ori.Printf(l->getSelRowText(8,AncienneSel)+wxT("%c")+l->getSelRowText(0,AncienneSel),SEPARATOR);
            prog=((h*1.0)/(l->GetSelectedItemCount()*1.0))*100.0;
            wxCommandEvent event( wxEVT_COMMAND_MENU_SELECTED, ID_THREAD);
            event.SetInt(-1);
            wxPostEvent( mb, event );

            if(mv){
                if(!wxRenameFile(path_ori,path))
                    throw 1;
                wxRmdir(l->getSelRowText(8,AncienneSel));//ne supprime que les dossiers vides
            }else{
                if(!wxCopyFile(path_ori,path))
                    throw 1;
            }
        }catch(...){
            long index = l->FindItem(-1,l->getSelRowText(0,AncienneSel));
            l->SetItemBackgroundColour(index,wxColour(220, 170, 170));
            errors ++;

        }
        AncienneSel = l->getSelection(AncienneSel);
    }

    return NULL;
}

long CopyMoveThread::getProg(){
    return prog;
}

int CopyMoveThread::getErrors(){
    return errors;
}
